var baseServerApi = function ($) {
	'use strict';

    //debugger;

    function makeURL(path, queryParams) {
        //debugger;
		var queryString = queryParams
			? '&' + $.param(queryParams)
			: '';
		return 'http://10.64.1.76:8080/api/' + path + '?AuditSource=WebDesigner' + queryString;
	}

	function getDefaultHeaders(token, enableCache) {
		var headers = {
			AuthToken: token,
			Accept: 'application/json'
		};
		if (!enableCache) {
			headers['Cache-Control'] = 'no-cache, no-store, must-revalidate';
			headers.Pragma = 'no-cache';
			headers.Expires = 0;
		}
		return headers;
	}

	/**
	 * Returns list with shared images
	 * @param {string} token auth token
	 */
	function getImagesList(token) {
		return $.getJSON({
			url: makeURL('images'),
			headers: getDefaultHeaders(token)
		});
	}

	function getImageUrl(options) {
		return makeURL('images/' + options.id + '/content', options.queryParams);
	}

	/**
	 * Returns list with shared data sources
	 * @param {string} token auth token
	 */
	function getDataSourcesList(token) {
		return $.getJSON({
			url: makeURL('datasources'),
			headers: getDefaultHeaders(token)
		});
	}

	/**
	 *  Returns data source query schema
	 * @param {string} token auth token
	 * @param {object} options
	 * @param {string} options.datasourceName data source name
	 * @param {string} options.commandText data source command text
	 * @param {string} options.commandType data source command type
	 * @param {Array<Object>} options.commandType list with data source parameters
	 */
	function getDataSetQuerySchema(token, options) {
		return $.ajax({
			url: makeURL('datasources/' + options.datasourceName + '/query/schema'),
			type: 'POST',
			data: JSON.stringify({
				commandText: options.commandText,
				commandType: options.commandType,
				parameters: options.parameters,
				timeout: 300000
			}),
			dataType: 'json',
			contentType: 'application/json',
			headers: getDefaultHeaders(token)
		});
	}

	/**
	 *  Returns result of the data source query text validation
	 * @param {string} token auth token
	 * @param {object} options
	 * @param {string} options.datasourceName data source name
	 * @param {string} options.commandText data source command text
	 * @param {string} options.commandType data source command type
	 * @param {Array<Object>} options.commandType list with data source parameters
	 */
	function validateDataSetQuery(token, options) {
		return $.ajax({
			url: makeURL('datasources/' + options.datasourceName + '/query/validate'),
			type: 'POST',
			data: JSON.stringify({
				commandText: options.commandText,
				commandType: options.commandType,
				parameters: options.parameters,
				timeout: 300000
			}),
			dataType: 'json',
			contentType: 'application/json',
			headers: getDefaultHeaders(token)
		});
	}

	/**
	 * Returns list with data source stored procedure parameters
	 * @param {string} token auth token
	 * @param {object} options
	 * @param {string} options.datasourceName data source name
	 * @param {string} options.commandText data source command text
	 * @param {string} options.commandType data source command type
	 */
	function getDataSetQueryStoredProcedureParameters(token, options) {
		return $.ajax({
			url: makeURL('datasources/' + options.datasourceName + '/storedprocedure/parameters'),
			type: 'POST',
			data: JSON.stringify({
				commandText: options.commandText,
				commandType: options.commandType,
				timeout: 300000
			}),
			dataType: 'json',
			contentType: 'application/json',
			headers: getDefaultHeaders(token)
		});
	}

	/**
	 * Returns site settings
	 * @param {string} token auth token
	 */
	function getSiteSettings(token) {
		return $.getJSON({
			url: makeURL('settings/site'),
			headers: getDefaultHeaders(token)
		});
	}

	function makeReportsSelector(allowedSemanticModelsId, hideFplReports) {
		var selector = {
			IsMaster: false,
			MasterReportId: { $in: [null] },
			$or: [
				{
					Type: 'SemanticReport'
				},
			],
			ModelId: {
				$in: [null].concat(allowedSemanticModelsId),
			},
		};
		var pageReportSelector = { Type: 'PageReport' };

		if (hideFplReports) pageReportSelector.IsCpl = true;
		selector.$or.push(pageReportSelector);

		return selector;
	}

	/**
	 * Returns list with reports
	 * @param {string} token auth token
	 * @param {object} options
	 * @param {Array<sting>} options.allowedSemanticModelsId list with allowable semantic report models
	 * @param {boolean} options.allowedSemanticModelsId display whenever you do not need to receive fpl reports
	 */
	function getReportsList(token, options) {
		var queryParams = {
			selector: JSON.stringify(makeReportsSelector(options.allowedSemanticModelsId, options.hideFplReports)),
		};
		return $.getJSON({
			url: makeURL('reports', queryParams),
			headers: getDefaultHeaders(token)
		});
	}

	/**
	 * Returns list with reports templates
	 * @param {string} token auth token
	 * @param {object} options
	 * @param {Array<sting>} options.allowedSemanticModelsId list with allowable semantic report models
	 * @param {boolean} options.allowedSemanticModelsId display whenever you do not need to receive fpl reports templates
	 */
	function getTemplatesList(token, options) {
		var queryParams = {
			selector: JSON.stringify(makeReportsSelector(options.allowedSemanticModelsId, options.hideFplReports)),
			domain: 'templates'
		};
		return $.getJSON({
			url: makeURL('reports', queryParams),
			headers: getDefaultHeaders(token)
		});
	}
	
	function getTemplateThumbnailUrl(options) {
		return makeURL('reports/' + options.id + '/thumbnail', options.queryParams);
	}

	/**
	 * Render images of chosen chart models
	 * @param {string} token auth token
	 * @param {Object} options
	 * @param {Object} options.chartsInfo chart models and some staff information
	 */
	function renderChartImages(token, options) {
		return $.ajax({
			type: 'POST',
			url: makeURL('designtime/chart'),
			data: JSON.stringify(options.chartsInfo),
			dataType: 'json',
			contentType: 'application/json',
			processData: true,
			headers: getDefaultHeaders(token)
		});
	}

	/**
	 * Get report content
	 * @param {string} token auth token
	 * @param {Object} options
	 * @param {string} options.id report id
	 */
	function getReportContent(token, options) {
		return $.getJSON({
			url: makeURL('reports/' + options.id + '/contentjson'),
			headers: getDefaultHeaders(token)
		});
	}

	/**
	 * Get list with report revisions
	 * @param {string} token auth token
	 * @param {Object} options
	 * @param {string} options.id report id
	 */
	function getReportRevisions(token, options) {
		return $.getJSON({
			url: makeURL('reports/' + options.id + '/versions'),
			headers: getDefaultHeaders(token)
		});
	}

	/**
	 * Save report as a new report
	 * @param {string} token auth token
	 * @param {Object} options
	 * @param {string} options.name report name
	 * @param {string} options.description report description
	 * @param {string} options.comment report revision comment
	 * @param {boolean} options.overwrite indicate whenever it is needed to overwrite saved report
	 * @param {boolean} options.temporary indicate whenever it is needed to save report as temporary report
	 * @param {boolean} options.isTemplate indicate whenever it is needed to save report as report template
	 */
	function saveNewReport(token, options) {
		var queryParams = {
			name: options.name,
			description: options.description,
			comment: options.comment,
			overwrite: options.overwrite,
			temporary: options.temporary,
			isTemplate: options.isTemplate
		};
		return $.ajax({
			type: 'POST',
			url: makeURL('reports/contentjson', queryParams),
			data: JSON.stringify(options.content),
			dataType: 'json',
			contentType: 'application/json',
			processData: false,
			headers: getDefaultHeaders(token)
		});
	}

	/**
	 * Save report as a new revision of an existing report
	 * @param {string} token auth token
	 * @param {Object} options
	 * @param {string} options.id report id
	 * @param {string} options.description report description
	 * @param {string} options.comment report revision comment
	 * @param {Object} options.content report content
	 * @param {boolean} options.isTemplate indicate whenever it is needed to save report as report template
	 */
	function saveExistingReport(token, options) {
		var queryParams = {
			comment: options.comment,
			isTemplate: options.isTemplate,
		};
		if (options.description !== null) queryParams.description = options.description;
		return $.ajax({
			type: 'PUT',
			url: makeURL('reports/' + options.id + '/contentjson', queryParams),
			data: JSON.stringify(options.content),
			dataType: 'json',
			contentType: 'application/json',
			processData: false,
			headers: getDefaultHeaders(token),
		});
	}

	/**
	 * Get list with data sets
	 * @param {string} token auth token
	 */
	function getDataSetsList(token) {
		return $.getJSON({
			url: makeURL('datasets'),
			headers: getDefaultHeaders(token),
		});
	}

	/**
	 * Get data set content
	 * @param {string} token auth token
	 * @param {Object} options
	 * @param {string} options.id data set id
	 */
	function getDataSetContent(token, options) {
		return $.getJSON({
			url: makeURL('datasets/' + options.id + '/contentjson'),
			headers: getDefaultHeaders(token),
		});
	}

	/**
	 * Get list with semantic models
	 * @param {string} token auth token
	 */
	function getSemanticModelsList(token) {
		return $.getJSON({
			url: makeURL('models'),
			headers: getDefaultHeaders(token),
		});
	}

	/**
	 * Get semantic model content
	 * @param {string} token auth token
	 * @param {Object} options
	 * @param {string} options.id semantic model id
	 * @param {string} options.version semantic model version
	 */
	function getSemanticModelContent(token, options) {
		var modelId = options.version ? (options.id + '.' + options.version) : options.id;
		return $.getJSON({
			url: makeURL('models/' + modelId + '/contentjson'),
			headers: getDefaultHeaders(token),
		});
	}

	/**
	 * Get semantic summary content
	 * @param {string} token auth token
	 * @param {Object} options
	 * @param {string} options.id semantic model id
	 * @param {string} options.version semantic model version
	 */
	function getSemanticModelSummary(token, options) {
		var modelId = options.version ? (options.id + '.' + options.version) : options.id;
		return $.getJSON({
			url: makeURL('models/' + modelId + '/contentjson', { summary: true }),
			headers: getDefaultHeaders(token),
		});
	}

	/**
	 * login
	 * @param {string} user
	 * @param {string} password
	 */
    function login(user, password) {
        //debugger;
		return $.ajax({
			url: makeURL('accounts/login'),
			type: 'POST',
			data: JSON.stringify({
				user: user,
				password: password,
			}),
			dataType: 'json',
			contentType: 'application/json',
		});
	}

	/**
	 * Get user info
	 * @param {string} token auth token
	 */
	function getUserInfo(token) {
		return $.getJSON({
			url: makeURL('user'),
			headers: getDefaultHeaders(token),
		});
	}

	/**
	 * Get accounts check result
	 * @param {string} token auth token
	 */
	function checkAuthToken(token) {
		return $.getJSON({
			url: makeURL('accounts/check'),
			headers: getDefaultHeaders(token),
		});
	}

	/**
	 * Get list with licenses
	 * @param {string} token auth token
	 */
	function getLicenses(token) {
		return $.getJSON({
			url: makeURL('licenses'),
			headers: getDefaultHeaders(token),
		});
	}

	/**
	 * Get server info
	 * @param {string} token auth token
	 */
	function getServerInfo(token) {
		return $.getJSON({
			url: makeURL('serverinfo'),
			headers: getDefaultHeaders(token),
		});
	}

	/**
	 * Get list with themes
	 * @param {string} token auth token
	 */
	function getThemesList(token) {
		return $.getJSON({
			url: makeURL('themes'),
			headers: getDefaultHeaders(token),
		});
	}

	/**
	 * Get theme content
	 * @param {string} token auth token
	 * @param {Object} options
	 * @param {string} options.id theme id
	 */
	function getThemeContent(token, options) {
		return $.getJSON({
			url: makeURL('themes/' + options.id + '/contentjson'),
			headers: getDefaultHeaders(token),
		});
	}

	/**
	 * Get list with styles
	 * @param {string} token auth token
	 */
	function getStylesList(token) {
		return $.getJSON({
			url: makeURL('reportstyles'),
			headers: getDefaultHeaders(token),
		});
	}

	/**
	 * Get style content
	 * @param {string} token auth token
	 * @param {Object} options
	 * @param {string} options.id style id
	 */
	function getStyleContent(token, options) {
		return $.getJSON({
			url: makeURL('reportstyles/' + options.id + '/contentjson'),
			headers: getDefaultHeaders(token),
		});
	}

	return {
		login: login,
		checkAuthToken: checkAuthToken,
		getLicenses: getLicenses,
		getServerInfo: getServerInfo,
		getSiteSettings: getSiteSettings,
		getUserInfo: getUserInfo,

		getReportsList: getReportsList,
		getReportRevisions: getReportRevisions,
		getReportContent: getReportContent,

		saveNewReport: saveNewReport,
		saveExistingReport: saveExistingReport,

		getTemplatesList: getTemplatesList,
		getTemplateThumbnailUrl: getTemplateThumbnailUrl,

		getSemanticModelsList: getSemanticModelsList,
		getSemanticModelContent: getSemanticModelContent,
		getSemanticModelSummary: getSemanticModelSummary,

		getDataSourcesList: getDataSourcesList,

		getDataSetsList: getDataSetsList,
		getDataSetContent: getDataSetContent,

		getThemesList: getThemesList,
		getThemeContent: getThemeContent,

		getStylesList: getStylesList,
		getStyleContent: getStyleContent,

		getImagesList: getImagesList,
		getImageUrl: getImageUrl,

		validateDataSetQuery: validateDataSetQuery,
		getDataSetQueryStoredProcedureParameters: getDataSetQueryStoredProcedureParameters,
		getDataSetQuerySchema: getDataSetQuerySchema,

		renderChartImages: renderChartImages,
	};
}(jQuery);
